/********************************************************
  Xpress-BCL C# Example Problems
  ==============================

  file foliomip2.cs
  `````````````````
  Modeling a small MIP problem
  to perform portfolio optimization.
   -- Imposing a minimum investment per share --

  (c) 2008-2024 Fair Isaac Corporation
      authors: S.Heipcke, D.Brett.
********************************************************/

using System;
using System.Text;
using System.IO;
using BCL;


namespace Examples
{

    public class TestUGFolioMip2
    {
        const int NSHARES = 10;                   // Number of shares
        const int NRISK = 5;                      // Number of high-risk shares
        const int NNA = 4;                        // Number of North-American shares

        double[] RET = {5,17,26,12,8,9,7,6,31,21};  // Estimated return in investment
        int[] RISK = {1,2,3,8,9};            // High-risk values among shares
        int[] NA = {0,1,2,3};                // Shares issued in N.-America

        public static void Main()
        {
            XPRB.init();
            int s;
            XPRBprob p = new XPRBprob("FolioSC");              // Initialize a new problem in BCL
            XPRBexpr Risk,Na,Return,Cap;
            XPRBvar[] frac = new XPRBvar[NSHARES];              // Fraction of capital used per share
            TestUGFolioMip2 TestInstance = new TestUGFolioMip2();

            // Create the decision variables
            for(s=0;s<NSHARES;s++)
            {
                frac[s] = p.newVar("frac", BCLconstant.XPRB_SC, 0, 0.3);
                frac[s].setLim(0.1);
            }

            // Objective: total return
            Return = new XPRBexpr();
            for (s = 0; s < NSHARES; s++) Return += TestInstance.RET[s] * frac[s];
            p.setObj(p.newCtr("Objective", Return));                   // Set the objective function

            // Limit the percentage of high-risk values
            Risk = new XPRBexpr();
            for (s = 0; s < NRISK; s++) Risk += frac[TestInstance.RISK[s]];
            p.newCtr(Risk <= 1.0/3);

            // Minimum amount of North-American values
            Na = new XPRBexpr();
            for (s = 0; s < NNA; s++) Na += frac[TestInstance.NA[s]];
            p.newCtr(Na >= 0.5);

            // Spend all the capital
            Cap = new XPRBexpr();
            for(s=0;s<NSHARES;s++) Cap += frac[s];
            p.newCtr(Cap == 1);

            // Solve the problem
            p.setSense(BCLconstant.XPRB_MAXIM);
            p.mipOptimize();

            // Solution printing
            System.Console.WriteLine("Total return: " + p.getObjVal());
            for(s=0;s<NSHARES;s++)
                System.Console.WriteLine(s + ": " + frac[s].getSol()*100 + "%");

            return;
        }

    }

}